package cz.wifisearch.checkDatabase;

import cz.wifisearch.domain.InputJson;
import cz.wifisearch.entity.DeviceE;
import cz.wifisearch.repository.DeviceRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created by Martin on 30.8.2015.
 * <p/>
 * Třída kontrolující zda se již dané zařízení nachází v databázi. Pokud ano, tak se provede pouze update hodnot.
 * Pokud zařízení není v databázi, tak se přidá a nastaví se příslušné hodnoty.
 */
public class CheckDevice {

    private DeviceRepository deviceRepository;
    private InputJson input;
    private DeviceE dev;
    private Logger logger = LoggerFactory.getLogger(CheckDevice.class);

    public CheckDevice(DeviceRepository deviceRepository, InputJson input) {
        this.deviceRepository = deviceRepository;
        this.input = input;
        check();
    }

    private void check() {
        /**
         * zjisteni, zda se znacka (vyrobce) vyskytuje v db
         */
        if (deviceRepository.countByBrand(input.getBrand()) == 0) {
            logger.info("insert new brand to db");
            saveDeviceInDB(input);
            deviceRepository.save(dev);
        } else {
            logger.info("update branch in database");
            // zjisteni zda je model v db
            if (deviceRepository.countByCodeNamed(input.getCodeNamed()) == 0) {
                saveDeviceInDB(input);
                deviceRepository.save(dev);
            } else {
                // zjisteni zda je v db prislusne sdk
                if (deviceRepository.countBySdk(input.getSdk()) == 0) {
                    saveDeviceInDB(input);
                    deviceRepository.save(dev);
                } else {
                    DeviceE deviceE = deviceRepository.findByCodeNamedAndSdk(input.getCodeNamed(), input.getSdk());
                    deviceE.setCountDev(deviceE.getCountDev() + 1);
                    deviceRepository.save(deviceE);
                }
            }
        }

    }

    private DeviceE saveDeviceInDB(InputJson input) {
        dev = new DeviceE();
        dev.setBrand(input.getBrand());
        dev.setCodeNamed(input.getCodeNamed());
        dev.setModel(input.getModel());
        dev.setManufactured(input.getManufactured());
        dev.setSdk(input.getSdk());
        dev.setReleaseSDK(input.getReleaseSDK());
        dev.setCountDev(1);
        return dev;
    }
}
