package cz.wifisearch.checkDatabase;

import cz.wifisearch.domain.InputJson;
import cz.wifisearch.domain.Wifi;
import cz.wifisearch.entity.PointE;
import cz.wifisearch.entity.WifiE;
import cz.wifisearch.repository.PointRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Martin on 25.8.2015.
 * <p/>
 * Třída kontrolující zda se již daný bod s příslušnými wifi nachází v databázi. Pokud ano a obsahuje všechny zjištěné
 * wifi tak se provede pouze update, pokud se vyskytuje nějaká novější wifi, tak se přidá do databáze. POkud ne, tak se
 * daný bod vloží.
 */
public class CheckPoint extends CompareLevel {

    private Logger logger = LoggerFactory.getLogger(CheckPoint.class);
    private PointRepository pointRepository;
    private InputJson input;

    private PointE currentPoint;
    private List<WifiE> listOfWifi;
    private WifiE wifi;
    private List<Wifi> inWifiList;
    private PointE p;

    public CheckPoint(PointRepository pointRepository, InputJson input) {
        this.pointRepository = pointRepository;
        this.input = input;
        inWifiList = input.getAvailableWifi();
        currentPoint = new PointE();
        check();
    }

    public void check() {
        if (pointRepository.findByNumberPoint(input.getIdPoint()) != null) {
            logger.info("update point in database");
            int countPointScan = pointRepository.findByCountScan(input.getIdPoint());
            PointE pointE = null;
            listOfWifi = new ArrayList<>();
            for (Wifi anInWifiList : inWifiList) {
                if (pointRepository.countAccessPoint(anInWifiList.getAccessPoint(), input.getIdPoint()) == 0) {
                    logger.info("BSSIS is not found in db");
                    logger.info("insert BSSID to db");
                    pointE = pointRepository.findByNumberPoint(input.getIdPoint());

                    List<WifiE> newWifi = new ArrayList<>();

                    setupWifiEntity(anInWifiList.getName(), anInWifiList.getAccessPoint(), anInWifiList.getCapabilities(),
                            anInWifiList.getLevel(), anInWifiList.getChannel(), anInWifiList.getFrequency(),
                            anInWifiList.getLevel(), anInWifiList.getLevel(), anInWifiList.getLevel(), 1, pointE);

                    newWifi.add(wifi);
                    pointE.setWifiE(newWifi);
                } else {
                    logger.info("BSSID is found in DB");
//                    double sum = pointRepository.findBySumLevel(anInWifiList.getAccessPoint(), input.getIdPoint());
//                    int countWifiScan = pointRepository.findByCountWifiScan(anInWifiList.getAccessPoint(), input.getIdPoint());
                    max = pointRepository.findMaxLevel(anInWifiList.getAccessPoint(), input.getIdPoint());
                    min = pointRepository.findMinLevel(anInWifiList.getAccessPoint(), input.getIdPoint());

                    compareMax(anInWifiList.getLevel());
                    compareMin(anInWifiList.getLevel());

                    pointE = pointRepository.findByAccessPoint(anInWifiList.getAccessPoint(), input.getIdPoint()); // nastaví se na null!!

                    WifiE wifiE = pointRepository.findWifi(input.getIdPoint(), anInWifiList.getAccessPoint());
                    wifiE.setName(anInWifiList.getName());
                    wifiE.setCapabilities(anInWifiList.getCapabilities());
                    wifiE.setLevel(anInWifiList.getLevel());
                    wifiE.setChannel(anInWifiList.getChannel());
                    wifiE.setFrequency(anInWifiList.getFrequency());
                    wifiE.setMaxLevel(max);
                    wifiE.setMinLevel(min);
//                    wifiE.setSumLevel(sum + anInWifiList.getLevel());
//                    wifiE.setCountScanWifi(countWifiScan + 1);
                    wifiE.setSumLevel(wifiE.getSumLevel() + anInWifiList.getLevel());
                    wifiE.setCountScanWifi(wifiE.getCountScanWifi() + 1);
                    wifiE.setPointE(pointE);

                    listOfWifi.add(wifiE);
                }
                // konec foreach
            }
            pointE.setWifiE(listOfWifi);
            pointE.setNumberPoint(input.getIdPoint());
            pointE.setCoordX(input.getCoordX());
            pointE.setCoordY(input.getCoordY());
            pointE.setCountScan(countPointScan + 1);
            pointE.setBuilding(input.getBuilding());
            pointE.setFloor(input.getFloor());
            pointE.setSignification(input.getSignification());
            pointE.setArea(input.getArea());
            pointE.setcDoors(input.getCountDoors());
            pointE.setPriority(input.getPriority());
            pointE.setTyp(input.getTyp());

            pointRepository.save(pointE);
        } else {
            logger.info("insert new record to db");
            listOfWifi = new ArrayList<>();

            for (Wifi anInWifiList : inWifiList) {
                setupWifiEntity(anInWifiList.getName(), anInWifiList.getAccessPoint(), anInWifiList.getCapabilities(),
                        anInWifiList.getLevel(), anInWifiList.getChannel(), anInWifiList.getFrequency(),
                        anInWifiList.getLevel(), anInWifiList.getLevel(), anInWifiList.getLevel(), 1, currentPoint);

                listOfWifi.add(wifi);
            }
            currentPoint.setWifiE(listOfWifi);
            currentPoint.setCoordX(input.getCoordX());
            currentPoint.setCoordY(input.getCoordY());
            currentPoint.setCountScan(1);
            currentPoint.setNumberPoint(input.getIdPoint());
            currentPoint.setBuilding(input.getBuilding());
            currentPoint.setFloor(input.getFloor());
            currentPoint.setSignification(input.getSignification());
            currentPoint.setArea(input.getArea());
            currentPoint.setcDoors(input.getCountDoors());
            currentPoint.setPriority(input.getPriority());
            currentPoint.setTyp(input.getTyp());

            pointRepository.save(currentPoint); // ulozeni do db
        }
    }

    private WifiE setupWifiEntity(String name, String accessPoint, String capabilities, int level, int channel, int frequency, int max, int min, double sumLevel, int countWifiScan, PointE point) {
        wifi = new WifiE();
        wifi.setName(name);
        wifi.setAccessPoint(accessPoint);
        wifi.setCapabilities(capabilities);
        wifi.setLevel(level);
        wifi.setChannel(channel);
        wifi.setFrequency(frequency);
        wifi.setMaxLevel(max);
        wifi.setMinLevel(min);
        wifi.setSumLevel(sumLevel);
        wifi.setCountScanWifi(countWifiScan);
        wifi.setPointE(point);
        return wifi;
    }
}
