package cz.wifisearch.controller.admin;

import cz.wifisearch.entity.User;
import cz.wifisearch.enumeration.SecurityRole;
import cz.wifisearch.repository.UserRepository;
import cz.wifisearch.service.UserService;
import cz.wifisearch.validators.PasswordValidator;
import cz.wifisearch.validators.UserEmailValidator;
import cz.wifisearch.validators.UserNameValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.validation.Valid;
import java.util.List;

/**
 * Created by Martin on 21.9.2015.
 */

@Controller
public class UserController {

    @Autowired
    UserService userService;
    @Autowired
    private UserRepository userRepository;
    @Autowired
    private UserNameValidator userNameValidator;
    @Autowired
    private UserEmailValidator emailValidator;
    @Autowired
    private PasswordValidator passwordValidator;

    // all
    @RequestMapping(value = "/admin/users", method = RequestMethod.GET)
    public String selectData(Model model) {

        List<User> users = userRepository.findAll();

        model.addAttribute("users", users);
        model.addAttribute("newUserForm", new User());

        return "admin/user/users";
    }

    // Add
    @RequestMapping(value = "/admin/user/form", method = RequestMethod.GET)
    public String showAddUserForm(Model model) {
        model.addAttribute("newUserForm", new User());
        return "admin/user/form";
    }

    @RequestMapping(value = "/admin/user/form", method = RequestMethod.POST)
    public String addForm(
            @Valid @ModelAttribute("newUserForm") User newUserForm,
            BindingResult result, Model model) {
        //validace jmena
        userNameValidator.validate(newUserForm, result);
        //validace emailu
        emailValidator.validate(newUserForm, result);
        //validace hesla
        passwordValidator.validate(newUserForm, result);

        if (result.hasErrors()) {
            return "admin/user/form";
        }
        userService.addUser(newUserForm.getFirstname(), newUserForm.getSurname(), newUserForm.getUsername(), newUserForm.getEmail(), newUserForm.getPassword(), SecurityRole.ROLE_ADMIN);
        return "redirect:/admin/users";
    }

    // edit
    @RequestMapping(value = "/admin/user/editUser", method = RequestMethod.GET)
    public String showEditUserForm(Model model) {

        String username = User.getCurrentUser();
        User currentUser = userRepository.findUserByUsername(username);

        User userForm = new User();
        userForm.setFirstname(currentUser.getFirstname());
        userForm.setSurname(currentUser.getSurname());
        userForm.setPassword(null);
        userForm.setEmail(currentUser.getEmail());

        model.addAttribute("editForm", userForm);
        return "admin/user/editUser";
    }

    @RequestMapping(value = "/admin/user/editUser", method = RequestMethod.POST)
    public String editUser(
            @Valid @ModelAttribute("editForm") User newUserForm,
            BindingResult result, Model model) {

        String name = User.getCurrentUser();
        User currentUser = userRepository.findUserByUsername(name);
        newUserForm.setUserId(currentUser.getUserId());
        newUserForm.setUsername(name);
        //validace jmena
        userNameValidator.validate(newUserForm, result);
        //validace emailu
        emailValidator.validate(newUserForm, result);
        //validace hesla
        passwordValidator.validate(newUserForm, result);

        if (result.hasErrors()) {
            return "admin/user/editUser";
        }

        userService.editUser(newUserForm.getFirstname(), newUserForm.getSurname(), newUserForm.getEmail(), newUserForm.getPassword());
        return "redirect:/admin/users";
    }

    @RequestMapping(value = "/admin/users/{id}", method = RequestMethod.GET)
    public String userDelete(@PathVariable int id) {
        userService.deleteUser(id);
        return "redirect:/admin/users";
    }

}
