package cz.wifisearch.controller.web;

import cz.wifisearch.checkDatabase.CheckDevice;
import cz.wifisearch.checkDatabase.CheckOperator;
import cz.wifisearch.checkDatabase.CheckPoint;
import cz.wifisearch.domain.InputJson;
import cz.wifisearch.domain.Position;
import cz.wifisearch.entity.PointE;
import cz.wifisearch.position.CalculateDistance;
import cz.wifisearch.repository.DeviceRepository;
import cz.wifisearch.repository.OperatorRepository;
import cz.wifisearch.repository.PointRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * Created by Martin on 8.8.2015.
 */

/*
 * @Controller anotace definuje třídu jako controllor. Controllor třídy ve
 * springu se používají k přípravě modelů, které budou namapovány na zobrazení
 * pro konkrétní zdroj.
 */
@Controller
public class JsonController {

    @Autowired
    private PointRepository pointRepository;

    @Autowired
    private OperatorRepository operatorRepository;

    @Autowired
    private DeviceRepository deviceRepository;

    private CheckPoint checkPoint;
    private CheckOperator checkOperator;
    private CheckDevice checkDevice;
    private CalculateDistance valueWithDB;
    private Position position;

    /* @RequestMapping slouží k mapování HTTP požadavku k určitému zdroji. */
    @RequestMapping(value = "/json", method = RequestMethod.POST, consumes = "application/json")
    public void addWifiPoint(@RequestBody InputJson input) { // uložení bodu do db

        checkPoint = new CheckPoint(pointRepository, input);// zjisteni zda se dany bod jiz nachazi v databazi

        String operator = input.getOperator();
        if (!operator.isEmpty()) // pokud je operator, ulozit, kvuli zařízení bez SIM
            checkOperator = new CheckOperator(operatorRepository, input); // zjisteni, zda se operator nachazi v db

        checkDevice = new CheckDevice(deviceRepository, input); // zjištění, zda se zařízení nachází v db

    }

    /* @RequestMapping slouží k mapování HTTP požadavku k určitému zdroji. */
    @RequestMapping(value = "/wifi", method = RequestMethod.POST, consumes = "application/json")
    @ResponseBody // aby to vytvorilo json s vyslednymi daty
    public ResponseEntity<Position> getWifi(@RequestBody InputJson input) { // zjištění wifi kvůli porovnání a určení pozice
        valueWithDB = new CalculateDistance(pointRepository, input);
        PointE pointE = valueWithDB.nearestPoint();
        return new ResponseEntity<>(createPosition(pointE), HttpStatus.OK);

    }

    private Position createPosition(PointE pointE) {
        position = new Position();
        position.setSignification(pointE.getSignification());
        position.setFloor(pointE.getFloor());
        position.setPriority(pointE.getPriority());
        position.setNumberPoint(pointE.getNumberPoint());
        position.setCoordX(pointE.getCoordX());
        position.setCoordY(pointE.getCoordY());
        position.setArea(pointE.getArea());
        position.setBuilding(pointE.getBuilding());
        position.setcDoors(pointE.getcDoors());
        position.setTyp(pointE.getTyp());
        return position;
    }

}
