package cz.wifisearch.position;

import cz.wifisearch.domain.InputJson;
import cz.wifisearch.entity.PointE;
import cz.wifisearch.repository.PointRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Martin on 18.9.2015.
 * <p/>
 * Třída porovnávající hodnoty ze vstupu Json - po stisknutí tlačítka pozice s otisky, které se nacházejí v databázi.
 * Pokud se najde shoda, tak se změní hodnota z -99 na hodnotu příslušného otisku. Když je nalezean shoda hodnota ze vstupu odsraněna,
 * aby docházelo k rychlejšímu prohledávání listu. Po projití celého listu se vypočítává eucleidova vzdálenost, podle které
 * se zjistí, který bod je nejblíže.
 */
public class CalculateDistance {
    private Logger logger = LoggerFactory.getLogger(CalculateDistance.class);
    private PointRepository pointRepository;
    private InputJson inputJson;

    public CalculateDistance(PointRepository pointRepository, InputJson inputJson) {
        this.pointRepository = pointRepository;
        this.inputJson = inputJson;
    }

    /**
     * Metoda která vybere všechny body (jejich ID) z databáze a u každého bodu provede kontrolu, zda odpovídají aktuální sitě
     * se záznamy v db. Pokud ano, tak se přepočítá distance. Následně se vybere bod s nejnižší distance.
     * @return point
     */
    public PointE nearestPoint() {
        List<Integer> allId = pointRepository.allNumberPoints();
        PointE point = new PointE();

        double value = 1000;
        double distance;
//        long numberId;

        for (Integer id : allId) {
            distance = 0;
            double result;

//            numberId = currentId;
//            List<String> p = pointRepository.findAccessPointByNumberPoint((int) numberId);  // zjistění názvu accessPoint z db
            List<String> accessPoint = pointRepository.findAccessPointByNumberPoint(id);  // zjistění názvu accessPoint z db

            logger.info("number of wireless networks " + inputJson.getAvailableWifi().size());
            // zjišení listu dostupných wifi
            for (int i = 0; i < inputJson.getAvailableWifi().size(); i++) {
                // zjistění názvu accessPoint ze vstupu
                String inputAccessPoint = inputJson.getAvailableWifi().get(i).getAccessPoint();
                // zjistění intenzity přislušnéhp accessPoint ze vstupu
                int inputLevel = inputJson.getAvailableWifi().get(i).getLevel();

                double dbAvgLevel = -99;
                for (String dbAccessPoint : new ArrayList<>(accessPoint)) {
                    if (inputAccessPoint.equals(dbAccessPoint)) {
//                        dbAvgLevel = pointRepository.findAvgIntensity((int) numberId, dbAccessPoint);
                        dbAvgLevel = pointRepository.findAvgIntensity(id, dbAccessPoint);
                        accessPoint.remove(dbAccessPoint);
                        break;
                    }
                }
                result = Math.pow(inputLevel - (dbAvgLevel), 2);
                distance += result;
            }
            double myDistance;
            myDistance = Math.sqrt(distance);

            if (myDistance < value) {
                value = myDistance;
//                int number = (int) numberId;
                logger.info("value " + value);
//                logger.info("number " + number);
                logger.info("id " + id);
//                point = pointRepository.findByNumberPoint(number);
                point = pointRepository.findByNumberPoint(id);
            }
        }
        return point;
    }
}