package cz.wifisearch.repository;

import cz.wifisearch.entity.PointE;
import cz.wifisearch.entity.WifiE;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * Created by Martin on 10.8.2015.
 */

@Repository
public interface PointRepository extends JpaRepository<PointE, Long> {

    // zda existuje BSSID v db
    @Query("select count(*) from PointE p JOIN p.wifiE w where w.accessPoint = ?1 and p.numberPoint = ?2")
    int countAccessPoint(String accessPoint,int numberPoint);

    //zjištení všeho z pointEntity, brany na základě numberPoint
    @Query("select p from PointE p where p.numberPoint = ?1")
    PointE findByNumberPoint(int numberPoint);

    @Query("select w.countScanWifi from PointE p JOIN p.wifiE w where w.accessPoint = ?1 and p.numberPoint = ?2")
    int findByCountWifiScan(String accessPoint,int numberPoint);// pocet skenovani

    @Query("select w.sumLevel from PointE p JOIN p.wifiE w where w.accessPoint = ?1 and p.numberPoint = ?2")
    double findBySumLevel(String accessPoint,int numberPoint);// najiti sumy

    @Query("select w.maxLevel from PointE p JOIN p.wifiE w where w.accessPoint = ?1 and p.numberPoint = ?2")
    int findMaxLevel(String accessPoint,int numberPoint);// maximální intenzita

    @Query("select w.minLevel from PointE p JOIN p.wifiE w where w.accessPoint = ?1 and p.numberPoint = ?2")
    int findMinLevel(String accessPoint,int numberPoint);// minimální intenzita

    @Query("select p.countScan from PointE p where p.numberPoint = ?1")
    int findByCountScan(int idPoint);// počet naskenování bodu

    @Query("select p from PointE p JOIN p.wifiE w where w.accessPoint = ?1 and p.numberPoint = ?2")
    PointE findByAccessPoint(String accessPoint,int numberPoint);// zjištení všeho z pointEntity, brany na základě accesspoint

    @Query("select w from PointE w order by w.idPointE desc")
    Page<PointE> findAll(Pageable pageable);

    @Query("select w, w.sumLevel/w.countScanWifi as avgLevel from PointE p JOIN p.wifiE w where p.numberPoint = ?1 order by avgLevel desc")
    Page<PointE> findAll(int numberPoint, Pageable pageable);

    @Query("SELECT p.numberPoint FROM PointE p order by p.numberPoint asc")
    List<Integer> allNumberPoints();

    @Query("select w.accessPoint from PointE p JOIN p.wifiE w where p.numberPoint = ?1")
    List<String> findAccessPointByNumberPoint(int numberPoint);

    @Query("select w.sumLevel/w.countScanWifi as avgLevel from PointE p JOIN p.wifiE w where p.numberPoint = ?1 and w.accessPoint = ?2")
    double findAvgIntensity(int numberPoint, String accessPoint);


    @Query("select w from PointE p JOIN p.wifiE w where p.numberPoint = ?1 and w.accessPoint = ?2")
    WifiE findWifi(int numberPoint, String accessPoint);

}
