package cz.wifisearch.service.impl;

import cz.wifisearch.entity.User;
import cz.wifisearch.entity.UserRole;
import cz.wifisearch.enumeration.SecurityRole;
import cz.wifisearch.repository.UserRepository;
import cz.wifisearch.repository.UserRoleRepository;
import cz.wifisearch.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;


// http://websystique.com/spring-security/spring-security-4-password-encoder-bcrypt-example-with-hibernate/

/**
 * Created by Martin on 21.9.2015.
 */
@Service
public class UserServiceImpl implements UserService {

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private UserRoleRepository userRoleRepository;

    @Transactional
    public User addUser(String firstname, String surname, String username,
                        String email, String password, SecurityRole role) {
        User user = new User();

        user.setFirstname(firstname);
        user.setSurname(surname);
        user.setUsername(username);
        user.setEmail(email);
        user.setPassword(password);
        user.setEnabled(true);

        UserRole ur = new UserRole();
        ur.setUser(user);
        ur.setRole(role);

        userRoleRepository.save(ur);

        return user;
    }


    @Transactional
    public User editUser(String firstname, String surname,
                         String email, String password) {
        String u = User.getCurrentUser();
        User user = userRepository.findUserByUsername(u);

        user.setFirstname(firstname);
        user.setSurname(surname);
        user.setEmail(email);
        user.setPassword(password);
        user.hashPassword();
        user.setEnabled(true);

//        UserRole ur = userRoleRepository.findOne((long) 1);
//        ur.setUser(user);

//        userRoleRepository.save(ur);
        userRepository.save(user);
        return user;
    }

    @Transactional
    public void disableAccount(User user) {
        user.setEnabled(false);
        userRepository.save(user);
    }


    @Override
    public boolean isUserNameInDatabase(String name, int userId) {
        if (name != null) {
            int count = userRepository.countUserNameInDatabase(name, userId);
            if (count != 0)
                return true;
            return false;
        } else
            return true;
    }

    @Override
    public boolean isEmailInDatabase(String email, int userId) {
        if (email != null) {
            int count = userRepository.countUserByEmailAndUserId(email, userId);
            if (count != 0)
                return true;
            return false;
        } else
            return true;
    }

    @Override
    @Transactional
    public void deleteUser(int id) {
        userRoleRepository.deleteRole(id);
        userRepository.deleteUser(id);
    }
}
